import * as mc from "@minecraft/server";
import { EquipmentSlot, GameMode } from "@minecraft/server";
import { GUN_PROPERTIES } from '../main/config/gun_type_config.js';
import { deadzone_advancements } from "./advancements.js";
import { getScore } from "./advancements.js";
import { showAvatarGui } from "./skins.js";
import { createCorpseOnDeath, despawnableEntities, despawnTimeSeconds, resetOnDeath, configUI, enableShowUiCreativeMod} from "config.js";
import { randomAvatar } from "./skins.js";
import { ActionFormData, MessageFormData } from "@minecraft/server-ui";

const activePlayers = new Map();
const bleedingIntervals = new Map();

mc.world.afterEvents.playerSpawn.subscribe(async (data) => {
    if (!data.initialSpawn) return;
    const player = data.player;

    try {
    	await player.runCommandAsync("execute as @s[tag=!join] run function DZgunsDataCount");
    	await player.runCommandAsync("execute as @s[tag=!join] run function DZammoStart");
        await player.runCommandAsync('tellraw @s {"rawtext":[{"translate":"game.message.welcome_dz"}]}');
        await player.runCommandAsync('tellraw @s {"rawtext":[{"translate":"game.message.deadzone_1"}]}');
        await player.runCommandAsync('tellraw @s {"rawtext":[{"translate":"game.message.deadzone_2"}]}');

        if (!player.hasTag('join')) {
            const randomTshirt = [
                "mcpe:tshirt_black",
                "mcpe:tshirt_blue",
                "mcpe:tshirt_red",
                "mcpe:tshirt_white",
                "mcpe:tshirt_green",
                "mcpe:tshirt_yellow"
            ];
            const selectedTshirt = randomTshirt[Math.floor(Math.random() * randomTshirt.length)];

            const randomJeans = [
                "mcpe:jean_black",
                "mcpe:jean_brown",
                "mcpe:jean_light",
                "mcpe:khaki_light"
            ];
            const selectedJeans = randomJeans[Math.floor(Math.random() * randomJeans.length)];

            const randomHats = [
                "mcpe:cap_blue",
                "mcpe:cap_green",
                "mcpe:cap_red",
                "mcpe:durag_black",
                "mcpe:durag_brown",
                "mcpe:flat_black",
                "mcpe:flat_brown",
                "mcpe:hunting_hat"
            ];
            const shouldGetHat = Math.random() < 0.5;
            const selectedHat = shouldGetHat ? randomHats[Math.floor(Math.random() * randomHats.length)] : null;

            const randomVests = [
                "mcpe:hunting_brown",
                "mcpe:hunting_navy",
                "mcpe:biker_jacket",
                "mcpe:traffic_lime",
                "mcpe:traffic_orange",
                "mcpe:press_vest"
            ];
            const shouldGetVest = Math.random() < 0.10;
            const selectedVest = shouldGetVest ? randomVests[Math.floor(Math.random() * randomVests.length)] : null;

            const randomFood = [
                "minecraft:apple",
                "mcpe:apple_green",
                "mcpe:pear"
            ];
            const shouldGetFood = Math.random() < 0.75;
            const selectedFood = shouldGetFood ? randomFood[Math.floor(Math.random() * randomFood.length)] : null;

            const randomMedical = [
                "mcpe:bandage"
            ];
            const shouldGetMedical = Math.random() < 0.55;
            const selectedMedical = shouldGetMedical ? randomMedical[Math.floor(Math.random() * randomMedical.length)] : null;

            const randomMelee = [
                "mcpe:baseball_bat",
                "mcpe:screwdriver",
                "mcpe:wooden_plank",
                "mcpe:hammer",
                "mcpe:wrench",
                "mcpe:pipe_wrench",
                "mcpe:hunting_knife"
            ];
            const shouldGetMelee = Math.random() < 0.15;
            const selectedMelee = shouldGetMelee ? randomMelee[Math.floor(Math.random() * randomMelee.length)] : null;

            const randomGuns = [
                {
                    gun: "mcpe:m1911",
                    ammo: "mcpe:ammo_45acp"
                },
                {
                    gun: "mcpe:makarov",
                    ammo: "mcpe:ammo_9x18mm"
                }
            ];
            const shouldGetGun = Math.random() < 0.05;
            const selectedGun = shouldGetGun ? randomGuns[Math.floor(Math.random() * randomGuns.length)] : null;
            
            await player.runCommandAsync(`give @s[tag=!join] ${selectedTshirt} 1`);
            await player.runCommandAsync(`give @s[tag=!join] ${selectedJeans} 1`);
            if (selectedHat) {
                await player.runCommandAsync(`give @s[tag=!join] ${selectedHat} 1`);
            }
            if (selectedVest) {
                await player.runCommandAsync(`give @s[tag=!join] ${selectedVest} 1`);
            }
            if (selectedFood) {
                await player.runCommandAsync(`give @s[tag=!join] ${selectedFood} 1`);
            }
            if (selectedMedical) {
                await player.runCommandAsync(`give @s[tag=!join] ${selectedMedical} 1`);
            }
            if (selectedMelee) {
                await player.runCommandAsync(`give @s[tag=!join] ${selectedMelee} 1`);
            }
            if (selectedGun) {
                await player.runCommandAsync(`give @s[tag=!join] ${selectedGun.gun} 1`);
                await player.runCommandAsync(`give @s[tag=!join] ${selectedGun.ammo} 1`);
            }
            
            await randomAvatar(player);
        }
    } catch (error) {
        console.error('Error: ', error);
    }
});

function scheduleDespawn(entity) {
    mc.system.runTimeout(() => {
        try {
            if (entity && entity.isValid()) {
                entity.triggerEvent("despawn");
            }
        } catch (e) {
        }
    }, despawnTimeSeconds * 20);
}

mc.world.afterEvents.entityDie.subscribe(async data => {
    const deadEntity = data.deadEntity;

    if (despawnableEntities.includes(deadEntity.typeId)) {
        if (createCorpseOnDeath) {
            deadEntity.triggerEvent("death");
        } else {
            deadEntity.triggerEvent("drop_item");
            deadEntity.triggerEvent("death_vanilla");
            mc.system.runTimeout(() => {
                deadEntity.triggerEvent("despawn");
            }, 2 * 20);
        }
        scheduleDespawn(deadEntity);
    }
    
    if (deadEntity.typeId === 'minecraft:player') {
    	await deadEntity.runCommandAsync(`scoreboard players set @s thirst 20`);
    	await deadEntity.runCommandAsync(`scoreboard players set @s blood 3500`);
    	await deadEntity.runCommandAsync(`scoreboard players set @s stamina 20`);
    	await deadEntity.runCommandAsync(`scoreboard players set @s infection 0`);
    }

    if (deadEntity.typeId === 'minecraft:player' && resetOnDeath) {
        resetPlayerScores(deadEntity);
    }
});


function resetPlayerScores(player) {
    const scores = ["player.kills", "zombie.kills", "time.playing", "survival.time", "longest.kill.distance", "scavenger.kills", "marauder.kills"];
    scores.forEach(async (score) => {
        await player.runCommandAsync(`scoreboard players set @s ${score} 0`);
    });
}

mc.world.afterEvents.entitySpawn.subscribe(entityNameTag);
function entityNameTag() {
	const entities = Array.from(mc.world.getDimension("overworld").getEntities());
	for (const entity of entities) {
		if (entity.typeId === "mcpe:scavenger") {
			entity.nameTag = "Scavenger";
		} else if (entity.typeId === "mcpe:marauder") {
			entity.nameTag = "Marauder";
		}
	}
}

function showAdvancementUI(player) {
    const form = new ActionFormData()
        .title("Advancement List")
        .body("Select an advancement to view details:");
    for (const key in deadzone_advancements) {
        const advancement = deadzone_advancements[key];
        const hasTag = player.hasTag(advancement.tags);
        const score = advancement.scoreRequired !== undefined ? getScore(advancement.objective, player, false) : undefined;

        if (!hasTag && (score === undefined || score < advancement.scoreRequired)) {
            form.button(`§c${advancement.name}`);
        } else {
            form.button(`§a${advancement.name}`);
        }
    }
    
    form.show(player).then((response) => {
        if (response.canceled) return;
        
        const advancementKey = Object.keys(deadzone_advancements)[response.selection];
        showAdvancementDetailUI(player, advancementKey);
    });
}

function showAdvancementDetailUI(player, key) {
    const advancement = deadzone_advancements[key];
    const form = new ActionFormData()
        .title(advancement.name)
        .body(`Description: §e${advancement.description} || "No description available."`);

    form.button("Back");
    form.show(player).then((response) => {
        if (response.canceled);
        showAdvancementUI(player);
    });
}

function getScore(objectiveName, player) {
    const objective = mc.world.scoreboard.getObjective(objectiveName);
    if (!objective) return 0;

    const participant = objective.getParticipants().find(p => p.displayName === player.nameTag);
    if (!participant) return 0;

    return objective.getScore(participant);
}

function setScore(objectiveName, player, value) {
    const objective = mc.world.scoreboard.getObjective(objectiveName);
    if (objective) {
        objective.setScore(player, value);
    }
}

function randomChance(percentage) {
    return Math.random() < percentage / 100;
}

function getThirstSymbol(thirst) {
    if (thirst >= 20) return ""; // full
    if (thirst >= 19) return ""; // full
    if (thirst >= 18) return ""; // full
    if (thirst >= 17) return ""; // full
    if (thirst >= 16) return ""; // full
    if (thirst >= 15) return ""; // full
    if (thirst >= 14) return ""; // full
    if (thirst >= 13) return ""; // full
    if (thirst >= 12) return ""; // full
    if (thirst >= 11) return ""; // full
    if (thirst >= 10) return ""; // half
    if (thirst >= 9) return ""; // half
    if (thirst >= 8) return ""; // half
    if (thirst >= 7) return ""; // half
    if (thirst >= 6) return ""; // half
    if (thirst >= 5) return ""; // half
    if (thirst >= 4) return ""; // half
    if (thirst >= 3) return ""; // half
    if (thirst >= 2) return ""; // half
    if (thirst >= 1) return ""; // half
    return ""; // 0
}

function getBloodSymbol(blood) {
    if (blood >= 1000) return ""; // full
    if (blood >= 900) return ""; // full
    if (blood >= 800) return ""; // full
    if (blood >= 700) return ""; // full
    if (blood >= 600) return ""; // full
    if (blood >= 500) return ""; // half
    if (blood >= 400) return ""; // half
    if (blood >= 300) return ""; // half
    if (blood >= 200) return ""; // half
    if (blood >= 100) return ""; // half
    return ""; // 0
}

function getInfectionSymbol(infection) {
    if (infection >= 100) return ""; // full
    if (infection >= 90) return ""; // half
    if (infection >= 80) return ""; // half
    if (infection >= 70) return ""; // half
    if (infection >= 60) return ""; // half
    if (infection >= 50) return ""; // half
    if (infection >= 40) return ""; // half
    if (infection >= 30) return ""; // half
    if (infection >= 20) return ""; // half
    return ""; // 0
}

function getStaminaSymbol(stamina) {
    if (stamina >= 20) return ""; // full
    if (stamina >= 19) return ""; // full
    if (stamina >= 18) return ""; // full
    if (stamina >= 17) return ""; // full
    if (stamina >= 16) return ""; // full
    if (stamina >= 15) return ""; // full
    if (stamina >= 14) return ""; // full
    if (stamina >= 13) return ""; // full
    if (stamina >= 12) return ""; // full
    if (stamina >= 11) return ""; // full
    if (stamina >= 10) return ""; // half
    if (stamina >= 9) return ""; // half
    if (stamina >= 8) return ""; // half
    if (stamina >= 7) return ""; // half
    if (stamina >= 6) return ""; // half
    if (stamina >= 5) return ""; // half
    if (stamina >= 4) return ""; // half
    if (stamina >= 3) return ""; // half
    if (stamina >= 2) return ""; // half
    if (stamina >= 1) return ""; // half
    return ""; // 0
}

function getBleedingLevel(player) {
	if (player.hasTag("bleeding10")) return 10;
	if (player.hasTag("bleeding9")) return 9;
	if (player.hasTag("bleeding8")) return 8;
	if (player.hasTag("bleeding7")) return 7;
	if (player.hasTag("bleeding6")) return 6;
	if (player.hasTag("bleeding5")) return 5;
	if (player.hasTag("bleeding4")) return 4;
    if (player.hasTag("bleeding3")) return 3;
    if (player.hasTag("bleeding2")) return 2;
    if (player.hasTag("bleeding1")) return 1;
    return 0;
}


function getAmmoSymbol(currentDurability, maxDurability) {
    if (currentDurability <= 0) return ""; // No ammo
    if (currentDurability <= maxDurability / 2) return ""; // Low ammo (Red)
    return ""; // Normal ammo (Green)
}

function updateDurabilityDisplay(item) {
    let durabilityComponent = item.getComponent("durability");
    let itemType = item.typeId;

    if (durabilityComponent && GUN_PROPERTIES[itemType]) {
        let maxDurability = GUN_PROPERTIES[itemType].durability;
        let currentDurability = maxDurability - durabilityComponent.damage;
        let ammoSymbol = getAmmoSymbol(currentDurability, maxDurability);
        let colorCode = currentDurability <= 0 ? "\xa74" : currentDurability <= maxDurability / 2 ? "\xa7c" : "\xa77";
        return `${ammoSymbol}: ${colorCode}${currentDurability}\xa7r/\xa7f${maxDurability}\xa7r`;
    }
    return "";
}

function updateThirst(player) {
    const thirst = getScore("thirst", player);
    const thirstPercentage = Math.floor((thirst / 20) * 100);
    const thirstSymbol = getThirstSymbol(thirst);

    player.onScreenDisplay.setTitle({
        rawtext: [
            { text: `uq${thirstSymbol}: §f${thirstPercentage}%%%` }
        ]
    });
}

function updateStamina(player) {
    const stamina = getScore("stamina", player);
    const staminaPercentage = Math.floor((stamina / 20) * 100);
    const staminaSymbol = getStaminaSymbol(stamina);

    player.onScreenDisplay.setTitle({
        rawtext: [
            { text: `ao${staminaSymbol}: §f${staminaPercentage}%%%` }
        ]
    });
}

function updateBlood(player) {
    const blood = getScore("blood", player);
    const bloodPercentage = Math.floor((blood / 3500) * 100);
    const bloodSymbol = getBloodSymbol(blood);

    player.onScreenDisplay.setTitle({
        rawtext: [
            { text: `bo${bloodSymbol}: ${bloodPercentage}%%%` }
        ]
    });
}

function updateAmmo(player) {
    const inventory = player.getComponent("inventory").container;
    const selectedItem = inventory.getItem(player.selectedSlotIndex);
    
    const gunDurability = selectedItem ? updateDurabilityDisplay(selectedItem) : "";
    
    player.onScreenDisplay.setTitle({
        rawtext: [
            { text: `mp${gunDurability}` }
        ]
    });
}

function updateCompass(player) {
    const equipmentInventory = player.getComponent("equippable");
    const mainHandItem = equipmentInventory.getEquipment(EquipmentSlot.Mainhand);
    const showPosition = mainHandItem && mainHandItem.typeId === "minecraft:compass";
    const positionText = showPosition ? `: ${Math.floor(player.location.x)}, ${Math.floor(player.location.y)}, ${Math.floor(player.location.z)}` : "";
        player.onScreenDisplay.setTitle({
        rawtext: [
            { text: `hf${positionText}` }
        ]
    });
}

function updateBleeding(player) {
    const bleedingLevel = getBleedingLevel(player);
    const showBleeding = bleedingLevel;
    const bleedingText = showBleeding ? `: §f${bleedingLevel}` : "";
    
    player.onScreenDisplay.setTitle({
        rawtext: [
            { text: `vj${bleedingText}` }
        ]
    });
}

function updateInfection(player) {
    const infection = getScore("infection", player);
    const infectionSymbol = getInfectionSymbol(infection);
    const infectionText = infection > 0 ? `${infectionSymbol}: §f${infection}` : "";
    
    player.onScreenDisplay.setTitle({
        rawtext: [
        	{ text: `cg${infectionText}` }
        ]
    });
}

// Thirst system
mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
        const gameMode = player.getGameMode();
        
        if (gameMode === GameMode.creative || gameMode === GameMode.spectator) return;

        const thirst = getScore("thirst", player);
        if (thirst > 0) {
            setScore("thirst", player, thirst - 1);
        } else {
            player.applyDamage(1);
        }
    });
}, 125 * 20);

// Stamina system
mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
        const gameMode = player.getGameMode();
        
        if (gameMode === GameMode.creative || gameMode === GameMode.spectator) return;

        const stamina = getScore("stamina", player);
        const isRunning = player.isSprinting;
        const isJumping = player.isJumping;
        const isSwimming = player.isSwimming;

        if (isRunning && isJumping) {
            if (stamina > 0) {
                setScore("stamina", player, stamina - 2);
            } else {
                player.runCommand(`effect @s slowness 3 2 true`);
                player.runCommand(`effect @s weakness 3 2 true`);
            }
        } else if (isRunning || isSwimming || isJumping) {
            if (stamina > 0) {
                setScore("stamina", player, stamina - 1);
            } else {
                player.runCommand(`effect @s slowness 3 2 true`);
                player.runCommand(`effect @s weakness 3 2 true`);
            }
        } else {
            if (stamina < 20) {
                setScore("stamina", player, stamina + 1);
            }
        }
    });
}, 3 * 20);

// Infection system
mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
        const gameMode = player.getGameMode();
        
        if (gameMode === GameMode.creative || gameMode === GameMode.spectator) return;

        const infection = getScore("infection", player);

        if (infection >= 25 && infection < 50) {
            player.runCommand(`effect @s hunger 9000 0 true`);
        } else if (infection >= 50 && infection < 75) {
            player.runCommand(`effect @s hunger 9000 1 true`);
        } else if (infection >= 75 && infection < 100) {
            player.runCommand(`effect @s hunger 9000 2 true`);
        } else if (infection >= 100) {
            player.runCommand(`kill @s`);
        }
    });
}, 125 * 20);

// Bleeding system
mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
        const gameMode = player.getGameMode();
        
        if (gameMode === GameMode.creative || gameMode === GameMode.spectator) return;

        const blood = getScore("blood", player);
        const bleedingLevel = getBleedingLevel(player);
        
        let bloodReduction = 0;
        if (bleedingLevel === 1) bloodReduction = 3;
        if (bleedingLevel === 2) bloodReduction = 7;
        if (bleedingLevel === 3) bloodReduction = 10;
        if (bleedingLevel === 4) bloodReduction = 14;
        if (bleedingLevel === 5) bloodReduction = 17;
        if (bleedingLevel === 6) bloodReduction = 21;
        if (bleedingLevel === 7) bloodReduction = 24;
        if (bleedingLevel === 8) bloodReduction = 27;
        if (bleedingLevel === 9) bloodReduction = 30;
        if (bleedingLevel === 10) bloodReduction = 35;
        
        const newBlood = Math.max(blood - bloodReduction, 0);
        setScore("blood", player, newBlood);
        
        if (bleedingLevel > 0 && bleedingLevel < 10) {
            const chance = bleedingLevel * 2;
            if (randomChance(chance)) {
            	
                player.removeTag(`bleeding${bleedingLevel}`);
                player.addTag(`bleeding${bleedingLevel + 1}`);
            }
        }
    });
}, 20);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
        const gameMode = player.getGameMode();
        
        if (gameMode === GameMode.creative || gameMode === GameMode.spectator) return;

        const bleedingLevel = getBleedingLevel(player);

        if (bleedingLevel === 3) {
            player.runCommand(`damage @s 1`);
        }
        if (bleedingLevel === 4) {
            player.runCommand(`damage @s 1`);
        }
        if (bleedingLevel === 5) {
            player.runCommand(`damage @s 2`);
        }
        if (bleedingLevel === 6) {
            player.runCommand(`damage @s 2`);
        }
        if (bleedingLevel === 7) {
            player.runCommand(`damage @s 3`);
        }
        if (bleedingLevel === 8) {
            player.runCommand(`damage @s 3`);
        }
        if (bleedingLevel === 9) {
            player.runCommand(`damage @s 4`);
        }
        if (bleedingLevel === 10) {
            player.runCommand(`damage @s 5`);
        }
    });
}, 200);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
        const gameMode = player.getGameMode();
        
        if (gameMode === GameMode.creative || gameMode === GameMode.spectator) return;

        const infection = getScore("infection", player);

        if (infection >= 75 && infection < 100) {
            player.runCommand(`damage @s 1`);
        }
    });
}, 200);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
        const gameMode = player.getGameMode();
        
        if (gameMode === GameMode.creative || gameMode === GameMode.spectator) return;

        const blood = getScore("blood", player);

        if (blood == 0) {
            player.runCommand(`kill @s`);
        }
    });
}, 20);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
        if (player.hasTag("bleeding1")) {
            const chance = randomChance(7); 
            if (chance) {
                player.removeTag("bleeding1");
                player.addTag("bleeding2");
            }
        } else if (player.hasTag("bleeding2")) {
            const chance = randomChance(7);
            if (chance) {
                player.removeTag("bleeding2");
                player.addTag("bleeding3");
            }
        }
    });
}, 20);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
        if (player.hasTag("bleeding1") || player.hasTag("bleeding2") || player.hasTag("bleeding3")) {
            const timer = getScore("bleedingTimer", player);
            if (timer >= 6000) {
                player.removeTag("bleeding1");
                player.removeTag("bleeding2");
                player.removeTag("bleeding3");
                player.runCommand(`scoreboard players reset @s bleedingTimer`);
            } else {
                setScore("bleedingTimer", player, timer + 1);
            }
        }
    });
}, 20);

// play nature sounds
mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
        player.runCommand(`event entity @s mcpe:nature_sounds`);
    });
}, 160);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach(async (pl) => {
        if (pl.hasTag("avatar")) {
            showAvatarGui(pl);
            pl.runCommandAsync(`tag @s[tag=!avatar] remove avatar`);
        }
    });
}, 30);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach(async (player) => {
        if (player.hasTag("config")) {
            configUI(player);
        }
    });
}, 30);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach(async (player) => {
        if (player.hasTag("list")) {
            showAdvancementUI(player);
            player.runCommandAsync(`tag @s remove list`);
        }
    });
}, 30);

mc.system.runInterval(() => {
	const players = mc.world.getAllPlayers();
	players.forEach((source) => {
		source.runCommandAsync(`tag @s remove bleeding0`);
		source.runCommandAsync(`effect @s[scores={infection=..20}] hunger 0 0 true`);
		source.runCommandAsync(`scoreboard players set @s[scores={stamina=21..}] stamina 20`);
		source.runCommandAsync(`scoreboard players set @s[scores={stamina=..-1}] stamina 0`);
		source.runCommandAsync(`scoreboard players set @s[scores={thirst=21..}] thirst 20`);
		source.runCommandAsync(`scoreboard players set @s[scores={blood=3501..}] blood 3500`);
		source.runCommandAsync(`scoreboard players set @s[scores={infection=..-1}] infection 0`);
		source.runCommandAsync(`gamerule showcoordinates false`);
		source.runCommandAsync(`gamerule showdaysplayed true`);
		source.runCommandAsync(`gamerule sendcommandfeedback false`);
    });
}, 1);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
    	updateThirst(player);
    });
}, 1);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
   	 updateStamina(player);
    });
}, 2);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
    	updateBlood(player);
    });
}, 3);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
   	 updateAmmo(player);
    });
}, 4);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
   	 updateCompass(player);
    });
}, 5);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
   	 updateBleeding(player);
    });
}, 6);

mc.system.runInterval(() => {
    const players = mc.world.getAllPlayers();
    players.forEach((player) => {
   	 updateInfection(player);
    });
}, 7);